/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

/* Define the tracing macros */
#include "trace.h"

/* Define the ARP data structures */
#include "arp.h"

#include <oskit/net/ether.h>

#include "nilo.h"

static unsigned char eth_broadcast[6] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };

/* The arptable is maintained in network order. */
arptable_t arptable[MAX_ARP];

void
add_to_cache(ip4_t addr, u_int8_t *hwaddr, enum arp_entry a)
{
    NILO_TRACE_FUNCTION;
    
    arptable[a].ipaddr.num = addr.num;
//    memcpy(&arptable[a].ipaddr.num, &addr, sizeof(addr));
    memcpy(arptable[a].hwaddr, hwaddr, sizeof(arptable[0].hwaddr));
    
    NILO_TRACE_FUNCTION_RETURN;
}

u_int8_t *
lookup_addr(ip4_t addr)
{
    NILO_TRACE_FUNCTION;
    
    if (addr.num == INADDR_BROADCAST)
        return (eth_broadcast);
    if (addr.num == arptable[ARP_THEM].ipaddr.num)
        return (arptable[ARP_THEM].hwaddr);
    if (addr.num == arptable[ARP_MYSELF].ipaddr.num)
        return (arptable[ARP_MYSELF].hwaddr);
    return (0);
    
    NILO_TRACE_FUNCTION_RETURN;
}

void
fill_arp_entry (enum arp_entry arpentry)
{
    ip4_t destip;
    u_int32_t temp, i;
    static int32_t timeout = 0;   
    struct arprequest arpreq;

    NILO_TRACE_FUNCTION;

    /* See if we have a hardware address, to go with the IP number */
    for (i = 0; i<ETHER_ADDR_SIZE; i++) {
        if (arptable[arpentry].hwaddr[i])
            break;
    }
    
    /* Need to do arp request cause we don't have the mac addr */
    if (i == ETHER_ADDR_SIZE) {
        client_info->client_state = ARPING;
        arpreq.hwtype = htons(1);
        arpreq.protocol = htons(ETHERTYPE_IP);
        arpreq.hwlen = ETHER_ADDR_SIZE;
        arpreq.protolen = 4;
        arpreq.opcode = htons(ARPOP_REQUEST);

        /* setup the Client entry */
        memcpy(arpreq.shwaddr, arptable[ARP_CLIENT].hwaddr,
               ETHER_ADDR_SIZE);
        temp =  arptable[ARP_CLIENT].ipaddr.num;
        memcpy(&arpreq.sipaddr, &temp, 4);

        /* setup the Server entry */
        temp = arptable[arpentry].ipaddr.num;
        memcpy (&arpreq.tipaddr, &temp, sizeof(temp));
        memset((char *)arpreq.thwaddr, 0, ETHER_ADDR_SIZE);
#ifdef NILO_USE_TRACING
        dump_arp_table (&arptable);
        dump_arp_packet (&arpreq);
#endif
        
        timeout = currticks() + DEFAULT_TIMEOUT;

        dhcp_eth_transmit(arptable[ARP_CLIENT].hwaddr, eth_broadcast,
                          ETHERTYPE_ARP, sizeof(struct arprequest),
                          &arpreq);
        
        while (timeout > currticks() ||
               client_info->client_state != ARP_DONE) {
        }
        if (client_info->client_state != ARP_DONE)
            NILO_TRACE_MSG ("Timed out getting ARP reply\n");

    }    
    NILO_TRACE_FUNCTION_RETURN;
}






