/* 
   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef __ARP_H__
#define __ARP_H__

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

enum arp_entry {
    ARP_MYSELF,
    ARP_THEM,
    ARP_MAXENTRY
};

typedef struct {
    ip4_t ipaddr;
    u_int8_t hwaddr[6];
} arptable_t;

#define MAC_BCAST_ADDR  "\xff\xff\xff\xff\xff\xff"

struct arprequest {
    u_int16_t hwtype;        /* hardware type (must be ARPHRD_ETHER) */
    u_int16_t protocol;      /* protocol type (must be ETH_P_IP) */
    int8_t hwlen;            /* hardware address length (must be 6) */
    int8_t protolen;         /* protocol address length (must be 4) */
    u_int16_t opcode;        /* ARP opcode */
    u_int8_t shwaddr[6];     /* sender's hardware address */
    u_int8_t sipaddr[4];     /* sender's IP address */
    u_int8_t thwaddr[6];     /* target's hardware address */
    u_int8_t tipaddr[4];     /* target's IP address */
};

typedef struct arprequest arp_t;

#define ARPOP_REQUEST	1
#define ARPOP_REPLY	2

#define ARP_CLIENT	0
#define ARP_SERVER	1
#define ARP_GATEWAY	2
#define ARP_NS		3
#define ARP_ROOTSERVER	4
#define ARP_SWAPSERVER	5
#define MAX_ARP		ARP_SWAPSERVER+1

extern arptable_t arptable[MAX_ARP];

extern void 
add_to_cache(ip4_t addr, u_int8_t *hwaddr, enum arp_entry a);

extern unsigned char *
lookup_addr(ip4_t inaddr);

extern void
fill_arp_entry (enum arp_entry arpentry);


#endif /* __ARP_H__ */

