/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is the include generated by configuring */
#include "niloconfig.h"

#include "trace.h"

/* Define the debug functions that can print the data structures. */
#include "dump.h"

/* Define the standard BSD TFTP data types */
#include <oskit/freebsd/sys/types.h>
#include <net/if.h>
#include <arpa/tftp.h>
#include <net/if_arp.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <arpa/inet.h>

#include "nilo.h"
#include "bootp.h"

#include <string.h>

static int32_t xid;
static u_int32_t vend_magic = VM_RFC1048; /* FIXME: this was a char ? */

int32_t
fill_bootp_request(bootp_t *bootp_packet, int8_t *macaddr, u_int32_t xid)
{
    NILO_TRACE_FUNCTION;

    memset(bootp_packet, 0, sizeof(*bootp_packet));
    bootp_packet->bp_opcode = BOOTP_REQUEST;
    bootp_packet->bp_xid = xid;
    bootp_packet->bp_htype = 1;
    bootp_packet->bp_hlen = 6;
    memcpy(bootp_packet->bp_chaddr, macaddr, sizeof(bootp_packet->bp_chaddr));
    memcpy(bootp_packet->bp_options, &vend_magic, sizeof(vend_magic));
    bootp_packet->bp_options[4] = TAG_END;
    bootp_packet->bp_secs = 0;
    
#ifdef NILO_USE_TRACING
//    NILO_TRACE_MSG ("Bootp packet: %s\n", hexify_buffer (bootp_packet, sizeof(*bootp_packet)));

    dump_bootp_packet (bootp_packet);
#endif /* NILO_USE_TRACING */

    NILO_TRACE_FUNCTION_RETURN;
    return (sizeof(*bootp_packet));
}

