/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

/* Define the tracing macros */
#include "trace.h"

/* Define the TFTP data structures */
#include "tftp.h"

/* Define the BOOTP data structures */
#include "bootp.h"

/* Define the DHCP data structures */
#include "dhcp.h"

/* Define the ARP data structures */
#include "arp.h"

/* Define the  data structures */
#include "dump.h"

// #include <oskit/dev/ethernet.h>
// #include <oskit/dev/linux.h>
#include <net/if_arp.h>
#include <net/ether.h>

/* dump the contents of a BOOTP packet */
void
dump_bootp_packet(bootp_t *bpr)
{
    unsigned char *p;
    int i;
    
    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("bootp.bp_opcode: 0x%x\n", bpr->bp_opcode);
    NILO_TRACE_MSG ("bootp.bp_htype: 0x%x\n", bpr->bp_htype);
    NILO_TRACE_MSG ("bootp.bp_hlen: 0x%x\n", bpr->bp_hlen);
    NILO_TRACE_MSG ("bootp.bp_hops: 0x%x\n", bpr->bp_hops);
    NILO_TRACE_MSG ("bootp.xid: 0x%04x\n", bpr->bp_xid);
    NILO_TRACE_MSG ("bootp.bp_secs: 0x%02x\n", bpr->bp_secs);
    NILO_TRACE_MSG ("bootp.bp_flags: 0x%02x\n", bpr->bp_flags);

    NILO_TRACE_MSG ("bootp.bp_ciaddr: 0x%04x, (%s)\n", bpr->bp_ciaddr.num,
                    _inet_ntoa(bpr->bp_ciaddr.num));
    NILO_TRACE_MSG ("bootp.bp_yiaddr: 0x%04x, (%s)\n", bpr->bp_yiaddr.num,
                    _inet_ntoa(bpr->bp_yiaddr.num));
    NILO_TRACE_MSG ("bootp.bp_siaddr: 0x%04x, (%s)\n", bpr->bp_siaddr.num,
                    _inet_ntoa(bpr->bp_siaddr.num));
    NILO_TRACE_MSG ("bootp.bp_giaddr: 0x%04x, (%s)\n", bpr->bp_giaddr.num,
                    _inet_ntoa(bpr->bp_giaddr.num));

    NILO_TRACE_MSG ("bootp.bp_chaddr: %s\n", hexify_buffer (bpr->bp_chaddr, BP_CHADDR_LEN/2));

    NILO_TRACE_MSG ("bootp.bp_sname: %s\n", (*bpr->bp_sname) ? (char *)bpr->bp_chaddr: "none yet");
    NILO_TRACE_MSG ("bootp.bp_file: %s\n", (*bpr->bp_file) ? bpr->bp_file: "none yet");

#ifdef NILO_PXE_COMP
    union {
        u_int8_t d[BP_VEND_LEN];        /* vendor-specific area */
        struct {
            u_int8_t  magic[4];         /* magic number */
            u_int32_t flags;            /* flags/opcodes etc */
            u_int8_t  pad[56];          /* padding chars */
        } v;
    } bp_options;
#else
    NILO_TRACE_MSG ("bootp.bp_vend:");
    for (p = bpr->bp_options; p < &bpr->bp_options[200] && *p != TAG_END; ++p)
        NILO_TRACE_PRINTF (" 0x%02x", *p);
    NILO_TRACE_PRINTF ("\n");
#endif
    NILO_TRACE_FUNCTION_RETURN;
}

/* dump the contents of a DHCP packet */
void
dump_dhcp_packet(dhcp_t *dhcp_pkt)
{
    u_int8_t *p;
    char s[DHCP_SNAME_LEN];
    int i, j, len, endtag, num;
    
    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("Dumping a DHCP packet..\n");
    NILO_TRACE_PRINTF ("\tdhcp.bp_opcode: 0x%x\n", dhcp_pkt->bp_opcode);
    NILO_TRACE_PRINTF ("\tdhcp.bp_htype: 0x%x\n", dhcp_pkt->bp_htype);
    NILO_TRACE_PRINTF ("\tdhcp.bp_hlen: 0x%x\n", dhcp_pkt->bp_hlen);
    NILO_TRACE_PRINTF ("\tdhcp.bp_hops: 0x%x\n", dhcp_pkt->bp_hops);
    NILO_TRACE_PRINTF ("\tdhcp.xid: 0x%04x\n", dhcp_pkt->bp_xid);
    NILO_TRACE_PRINTF ("\tdhcp.bp_secs: 0x%02x\n", dhcp_pkt->bp_secs);
    NILO_TRACE_PRINTF ("\tdhcp.bp_flags: 0x%02x\n", dhcp_pkt->bp_flags);

    NILO_TRACE_PRINTF ("\tdhcp.bp_ciaddr: 0x%04x, (%s)\n",
                       dhcp_pkt->bp_ciaddr.num,
                    _inet_ntoa(dhcp_pkt->bp_ciaddr.num));
    NILO_TRACE_PRINTF ("\tdhcp.bp_yiaddr: 0x%04x, (%s)\n",
                       dhcp_pkt->bp_yiaddr.num,
                    _inet_ntoa(dhcp_pkt->bp_yiaddr.num));
    NILO_TRACE_PRINTF ("\tdhcp.bp_siaddr: 0x%04x, (%s)\n",
                       dhcp_pkt->bp_siaddr.num,
                    _inet_ntoa(dhcp_pkt->bp_siaddr.num));
    NILO_TRACE_PRINTF ("\tdhcp.bp_giaddr: 0x%04x, (%s)\n",
                       dhcp_pkt->bp_giaddr.num,
                    _inet_ntoa(dhcp_pkt->bp_giaddr.num));

    NILO_TRACE_PRINTF ("\tdhcp.bp_chaddr: %s\n",
                       hexify_buffer (dhcp_pkt->bp_chaddr, 
                                      (BP_CHADDR_LEN/2)-2));

    NILO_TRACE_PRINTF ("\tdhcp.bp_sname: %s\n",
                       (*dhcp_pkt->bp_sname && isascii(*dhcp_pkt->bp_sname))
                       ? (char *)dhcp_pkt->bp_sname: "none yet");
    NILO_TRACE_PRINTF ("\tdhcp.bp_file: %s\n", (*dhcp_pkt->bp_file)
                       ? dhcp_pkt->bp_file: "none yet");

    NILO_TRACE_PRINTF ("\tdhcp.bp_options:");

    /* Look for the last 0xff in the option field, cause it's the
       end tag */
    endtag = DHCP_MAX_OPT_SIZE;
    while (dhcp_pkt->bp_options[endtag--] != TAG_END) {    }

    for (p = dhcp_pkt->bp_options; p < &dhcp_pkt->bp_options[DHCP_MAX_OPT_SIZE] && p < (dhcp_pkt->bp_options + endtag); ++p)
        NILO_TRACE_PRINTF (" 0x%02x", *p);
    NILO_TRACE_PRINTF (" 0x%02x", *p);  /* print the end tag */
    NILO_TRACE_PRINTF ("\n");
    
    p = dhcp_pkt->bp_options;
    i = 4;                              /* ignore the vendor magic */

    while (p[i] != TAG_END && p[i] != 0) {
        NILO_TRACE_PRINTF ("\t\toption %s (0x%02x): ", optnames[p[i-1]], p[i++]);
        len = p[i++];
        switch (p[i-2]) {
        case DHCP_HOSTNAME:
        case DHCP_DOMAINNAME:
        case DHCP_CLIENT_IDENTIFIER:
            memcpy (s, (char *)p+i, len);
            s[len] = '\0';
            NILO_TRACE_PRINTF ("%s\n", s);
            break;
        case DHCP_REQUESTED_IP_ADDR:
        case DHCP_SUBNET_MASK:
        case DHCP_ROUTERS_ON_SUBNET:
        case DHCP_SERVER_IDENTIFIER:
        case DHCP_BROADCAST_ADDR:
            memcpy (&num, p+i, len);
            NILO_TRACE_PRINTF ("%s\n", _inet_ntoa(num));
            break;
        case DHCP_MESSAGE_TYPE:
            NILO_TRACE_PRINTF ("type %s (0x%02x): \n", msgtypes[p[i]], p[i]);
            break;
        default:
            for (j=0; j<len; j++)
                NILO_TRACE_PRINTF (" 0x%02x", p[i+j]);
            NILO_TRACE_PRINTF ("\n");
        }
        i+=len;
    }

    NILO_TRACE_FUNCTION_RETURN;
}


/* dump the contents of a TFTP packet */
void
dump_tftp_packet(tftp_t *pkt)
{
    unsigned char *p;
    int i;
    
    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("Dumping a TFTP packet..\n");

    switch (ntohs(pkt->opcode)) {
    case TFTP_RRQ:
        NILO_TRACE_PRINTF ("\ttftp.opcode: RRQ\n");
        NILO_TRACE_PRINTF ("\ttftp.u.data: %s\n", pkt->u.rrq);
        NILO_TRACE_PRINTF ("\ttftp mode is: %s\n",
                           (pkt->u.rrq+strlen(pkt->u.rrq)+1));
        break;
    case TFTP_WRQ:
        NILO_TRACE_PRINTF ("\ttftp.opcode: WRQ\n");
        NILO_TRACE_PRINTF ("\ttftp.u.data: %s\n", pkt->u.rrq);
        NILO_TRACE_PRINTF ("\ttftp mode is: %s\n",
                           (pkt->u.rrq+strlen(pkt->u.rrq)+1));
        break;
    case TFTP_ACK:
        NILO_TRACE_PRINTF ("\ttftp.opcode: ACK\n");
        NILO_TRACE_PRINTF ("\ttftp.u.ack.block: 0x%02x\n",
                           ntohs(pkt->u.ack.block));
        break;
    case TFTP_OACK:
        NILO_TRACE_PRINTF ("\ttftp.opcode: OACK\n");
        NILO_TRACE_MSG ("FIXME: we don't handle OACK packets yet.\n");
        break;
    case TFTP_DATA:
        NILO_TRACE_PRINTF ("\ttftp.opcode: DATA\n");
        NILO_TRACE_PRINTF ("\ttftp.data.block: 0x%02x\n",
                           ntohs(pkt->u.data.block));
        NILO_TRACE_PRINTF ("\ttftp.u.data: %s lots more...\n",
                           hexify_buffer(pkt->u.data.download, 30));

    }

    NILO_TRACE_FUNCTION_RETURN;
}

void
dump_ip_packet (struct iphdr *ip)
{
    u_int32_t num;

    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("Dumping an IP packet..\n");
    NILO_TRACE_PRINTF ("\tip.version: 0x%0x\n", ip->verhdrlen >> 4);
    NILO_TRACE_PRINTF ("\tip.hdrlen: 0x%0x\n", ip->verhdrlen & 0xf);
    NILO_TRACE_PRINTF ("\tip.service: 0x%02x\n", ip->service);
    NILO_TRACE_PRINTF ("\tip.len: 0x%04x\n", ntohs(ip->len));
    NILO_TRACE_PRINTF ("\tip.ident: 0x%04x\n", ntohs(ip->ident));
    NILO_TRACE_PRINTF ("\tip.frags: 0x%04x\n", ntohs(ip->frags));
    NILO_TRACE_PRINTF ("\tip.ttl: 0x%02x\n", ip->ttl);
    NILO_TRACE_PRINTF ("\tip.protocol: 0x%02x\n", ip->protocol);
    NILO_TRACE_PRINTF ("\tip.chksum: 0x%04x\n", ntohl(ip->chksum));
    memcpy (&num, ip->src, 4);
    NILO_TRACE_PRINTF ("\tip.src: 0x%04x (%s)\n", ntohl(num), _inet_ntoa(num));
    memcpy (&num, ip->dest, 4);
    NILO_TRACE_PRINTF ("\tip.dest: 0x%04x (%s)\n", ntohl(num), _inet_ntoa(num));
    
    NILO_TRACE_FUNCTION_RETURN;
}

void
dump_udp_packet (struct udphdr *udp)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("\tDumping a UDP packet..\n");

    NILO_TRACE_PRINTF ("\tudp.src:  0x%04x\n", ntohs(udp->src));
    NILO_TRACE_PRINTF ("\tudp.dest: 0x%04x\n", ntohs(udp->dest));
    NILO_TRACE_PRINTF ("\tudp.len: 0x%04x\n", ntohs(udp->len));
    NILO_TRACE_PRINTF ("\tudp.chksum: 0x%04x\n", udp->chksum);

    NILO_TRACE_FUNCTION_RETURN;
}

void
dump_arp_table(arptable_t *arptbl)
{
    int i;

    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("Dumping the ARP table\n");
    for (i=0; i<MAX_ARP; i++) {
        NILO_TRACE_PRINTF ("\t\t%s", arpentries[i]);
        NILO_TRACE_PRINTF ("\t%s", _inet_ntoa(arptbl[i].ipaddr));
        if (arptbl[i].ipaddr.num == 0)
            NILO_TRACE_PRINTF ("\t");
        NILO_TRACE_PRINTF ("\t\t%s\n", hexify_buffer (arptbl[i].hwaddr, 6));
    }

    NILO_TRACE_FUNCTION_RETURN;
}

/* Dump an ARP packet */
void
dump_arp_packet (struct arprequest *pkt)
{
    u_int32_t temp;

    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("Dumping an ARP packet...\n");

    NILO_TRACE_PRINTF ("\t\tarprequest.hwtype: 0x%02x\n", 
                           ntohs(pkt->hwtype));
    
    if (ntohs(pkt->protocol) == ETHERTYPE_IP)
        NILO_TRACE_PRINTF ("\t\tarprequest.protocol: 0x%02x\n",
                           ntohs(pkt->protocol));
    else
        NILO_TRACE_PRINTF ("\t\tBAD protocol type, 0x%02x! \n",
                           ntohs(pkt->protocol));
   
    if (pkt->hwlen == 6)
        NILO_TRACE_PRINTF ("\t\tarprequest.hwlen: 0x%01x\n",
                           pkt->hwlen);
    else
        NILO_TRACE_PRINTF ("\t\tBAD hardware address length!\n",
                           pkt->hwlen);
   
    if (pkt->protolen == 4)
        NILO_TRACE_PRINTF ("\t\tarprequest.protolen: 0x%01x\n",
                           pkt->protolen);
    else
        NILO_TRACE_PRINTF ("BAD protocol length, 0x%01x!\n",
                           pkt->protolen);
       
    NILO_TRACE_PRINTF ("\t\tarprequest.opcode: %s\n",
                       arpops[ntohs(pkt->opcode)]);

    NILO_TRACE_PRINTF ("\t\tarprequest.shwaddr: %s\n",
                       hexify_buffer (pkt->shwaddr, 6));

    memcpy(&temp, pkt->sipaddr, sizeof(temp));
    NILO_TRACE_PRINTF ("\t\tarprequest.sipaddr: 0x%04x, (%s)\n",
                       temp, _inet_ntoa(temp));
//                       pkt->sipaddr, _inet_ntoa(pkt->sipaddr));

    NILO_TRACE_PRINTF ("\t\tarprequest.thwaddr: %s\n",
                       hexify_buffer (pkt->thwaddr, 6));
    memcpy(&temp, pkt->tipaddr, sizeof(temp));
    NILO_TRACE_PRINTF ("\t\tarprequest.tipaddr: 0x%04x, (%s)\n",
                       temp, _inet_ntoa(temp));

    NILO_TRACE_FUNCTION_RETURN;
}


