/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

/* Define the tracing macros */
#include "trace.h"

/* Define the bootp data structures */
#include "bootp.h"

/* Define the data structure dumping functions for debug use */
#include "dump.h"

#include "nilo.h"

/* Define the standard BSD TFTP data types */
#include <oskit/freebsd/sys/types.h>
#include <net/if.h>
#include <arpa/tftp.h>
#include <net/if_arp.h>

#include <stdlib.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include "tagged.h"
#include "util.h"

#undef	SIMULATE_BLOCK_LOSS

extern u_int32_t jiffies;

struct client_data *client_info;
bootp_t *bootpr = 0;
bootp_t *dhcp_ptr = 0;
static tftp_t *tftpr = 0;
static int32_t timeout;
static u_int32_t xid;
#ifdef	SIMULATE_BLOCK_LOSS
static int32_t losethisblock = 0;
#endif

int32_t seed;	/* for random number generator */

// static void bootp_callback (int32_t snaplen, bootp_t *bootp_ptr);
static void dhcp_callback (int32_t snaplen, dhcp_t *dhcp_ptr);
// static void process_bootp_reply (bootp_t *bootp_ptr);
static void process_dhcp_reply (dhcp_t *dhcp_ptr);

/**************************************************************************
RFC951_SLEEP - sleep for expotentially longer times
**************************************************************************/
int32_t
rfc951_sleep(int32_t exp)
{
    int32_t q,z,tmo;

    NILO_TRACE_FUNCTION;

    /* simplified version of the LCG given in Bruce Scheier's
       "Applied Cryptography" */
    q = seed/53668;
    if ((seed = 40014*(seed-53668*q) - 12211*q) < 0) seed += 2147483563l;
    /* compute mask */
    for (tmo = 63; tmo <= 60*1008 && --exp > 0; tmo = 2*tmo+1);
    return (tmo & seed);

    NILO_TRACE_FUNCTION_RETURN;
}

static void
jump_to_code(void)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_FUNCTION_RETURN;
}

int 
main(int argc, char **argv)
{
    int32_t retries;
    struct in_addr addr;
    
    NILO_TRACE_FUNCTION;
    
    NILO_TRACE_PRINTF ("NILO dated " VERSION "\n");
    init_system();

    client_info = init_protocol_code();

    init_hardware(client_info);
    
    NILO_TRACE_MSG ("Sending dhcp request\n");

    /* Send a DHCPDISCOVER packet */
    client_info->dhcp_handler(client_info->xid = getxid());

    if (client_info->client_state == DISCOVERING)
        panic ("DHCP server is not responding");

    /* Send a DHCPREQUEST packet */
    client_info->dhcp_handler((long)client_info->xid = getxid());

    NILO_TRACE_MSG ("Got dhcp answer\n");

    /* Setup the ARP tables */
    fill_arp_entry (ARP_SERVER);
//    fill_arp_entry (ARP_GATEWAY);

    /* Start the TFTP transfer */
    client_info->tftp_handler(client_info->siaddr.num, TFTP_RRQ,
                          client_info->file, client_info->tftp_blockno);

    jump_to_code();

    NILO_TRACE_FUNCTION_RETURN;

    return (0);
}

void
bootp_callback (int32_t snaplen, bootp_t *bpr)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("Bootp reply received\n");
    if (bpr->bp_xid != client_info->xid)		/* not for us */
        return;
    bootpr = bpr;

    NILO_TRACE_FUNCTION_RETURN;
}

void
dhcp_callback (int32_t snaplen, dhcp_t *dptr)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("DHCP reply received\n");
    if (dptr->bp_xid != client_info->xid)		/* not for us */
        return;

#ifdef NILO_USE_TRACING
    dump_dhcp_packet (dptr);
#endif /* NILO_USE_TRACING */

//    dhcp_ptr = dptr;

    NILO_TRACE_FUNCTION_RETURN;
}

void
process_bootp_reply (bootp_t *bpr)
{
    unsigned char *p;
    
    NILO_TRACE_FUNCTION;

    for (p = bpr->bp_options; p < &bpr->bp_options[200] && *p != TAG_END; ++p)
        NILO_TRACE_MSG ("%02x ", *p);
//    add_to_cache(bpr->bp_yiaddr.num, bpr->bp_chaddr, 0);
/*    add_to_cache(bpr->bp_yiaddr.num, bpr->bp_chaddr, ARP_MYSELF); */

#ifdef NILO_USE_TRACING
    dump_bootp_packet (bpr);
#endif /* NILO_USE_TRACING */

    NILO_TRACE_FUNCTION_RETURN;
}

void
process_dhcp_reply (dhcp_t *dptr)
{
    unsigned char *p;
    
    NILO_TRACE_FUNCTION;

    for (p = dptr->bp_options; p < &dptr->bp_options[200] && *p != TAG_END; ++p)
        NILO_TRACE_MSG ("%02x ", *p);
//    add_to_cache(dptr->bp_yiaddr.num, dptr->bp_chaddr, 0);
/*    add_to_cache(dptr->bp_yiaddr.num, bpr->bp_chaddr, ARP_MYSELF); */

#ifdef NILO_USE_TRACING
    dump_dhcp_packet (dptr);
#endif /* NILO_USE_TRACING */

    NILO_TRACE_FUNCTION_RETURN;
}
