/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is the include generated by configuring */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE types */
#include "pxe_types.h"

/* Define the tracing macros */
#include "trace.h"

/* Define the bootp data structures */
#include "bootp.h"

/* Define the TFTP data structures */
#include "tftp.h"

/* Define the DHCP data structures */
#include "dhcp.h"

/* Define the UDP data structures */
#include "udp.h"

/* Define the ARP data structures */
#include "arp.h"

/* Define the internal Nilo data structures */
#include "nilo.h"

/* Define the network data structures */
#include "net.h"

#include <oskit/x86/pc/rtc.h>
#include <oskit/x86/pc/phys_lmm.h>
#include <oskit/x86/types.h>
#include <oskit/x86/base_cpu.h>
#include <oskit/x86/base_vm.h>
#include <oskit/x86/proc_reg.h>

#include <oskit/lmm.h>
#include <oskit/types.h>

#include <oskit/dev/dev.h>
#include <oskit/dev/net.h>
#include <oskit/dev/error.h>
#include <oskit/net/ether.h>
#include <oskit/dev/ethernet.h>
#include <oskit/dev/linux.h>
#include <oskit/dev/device.h>

#include <oskit/io/netio.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <malloc.h>
#include <errno.h>
#include <netdb.h>
#include <assert.h>

#include <oskit/c/sys/time.h>
#include <oskit/c/sys/socket.h>

void bumpticks(void);
volatile unsigned long ticks = 0;

static int32_t bootp_socket = -1, tftp_socket = -1;
static struct sockaddr_in bootp_server, bootp_client,
				tftp_server, tftp_client;
static dhcp_t bp_sndpkt;
static dhcp_t bp_rcvpkt;
static tftp_t tftp_pkt;

static void (*bootp_callback)(int32_t, bootp_t *);
static void (*dhcp_callback)(int32_t, bootp_t *);
static void (*tftp_callback)(int32_t, tftp_t *);

// static client_states_t client_state = REBOOTING;

unsigned long jiffies = 0L;

#define MAX_DEVICES 4			/* max number of ethernet cards */

static struct etherdev devs[MAX_DEVICES];

oskit_error_t
net_receive(void *data, oskit_bufio_t *b, oskit_size_t pkt_size);

void
init_system(void)
{
    NILO_TRACE_FUNCTION;
    
    /* Initialize the global data structure */
    memset((u_int8_t *)&client_info, 0, sizeof(client_info));

    NILO_TRACE_FUNCTION_RETURN;
}

/* init_hardware --
   Initialize the hardware.
   FIXME: This should either return a void, or a real 1 or 0 
   error message. */ 
int32_t
init_hardware(struct client_data *prot)
{
    char *p;
    oskit_error_t err;
    oskit_etherdev_t **etherdev;
    int i, a0, a1, a2, a3, a4, a5;
    
    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("Initializing devices...\n");
    oskit_dev_init();
    
    oskit_linux_init_net();
    oskit_dump_drivers();
    
    NILO_TRACE_MSG("Probing devices...\n");
    oskit_dev_probe();
    oskit_dump_devices();
    
    osenv_timer_register(bumpticks, BOOTP_TIMER_FREQ);

    /* Find all the Ethernet device nodes. */
    client_info->ndev = osenv_device_lookup(&oskit_etherdev_iid, (void***)&etherdev);
    if (client_info->ndev <= 0)
        panic("no Ethernet adaptors found!");
    if (client_info->ndev > MAX_DEVICES)
        panic("XXX too many Ethernet adaptors found");
    
    /* Open all the devices and have them hand packets to the OS. */
    NILO_TRACE_MSG("%d Ethernet adaptor%s found:\n", client_info->ndev,
                   client_info->ndev > 1 ? "s" : "");
    for (i = 0; i < client_info->ndev; i++) {
        int j;
        
        err = oskit_etherdev_getinfo(etherdev[i], &devs[i].info);
        if (err)
            panic("error getting info from ethercard %d", i);
        
        oskit_etherdev_getaddr(etherdev[i], devs[i].haddr);
        
        /* Show information about this adaptor */
        NILO_TRACE_PRINTF("  %-16s%-40s ", devs[i].info.name,
               devs[i].info.description
               ? devs[i].info.description : "");
        for (j = 0; j < 5; j++) 
            NILO_TRACE_PRINTF ("%02x:", devs[i].haddr[j]);
        NILO_TRACE_PRINTF("%02x\n", devs[i].haddr[5]);
        
        devs[i].dev = etherdev[i];
        devs[i].recv_nio = oskit_netio_create(net_receive, &devs[i]);
        if (devs[i].recv_nio == NULL)
            panic("unable to create recv_nio");
        
        err = oskit_etherdev_open(etherdev[i], 0, devs[i].recv_nio,
                                  &devs[i].send_nio);
    }
    
    if (!osenv_intr_enabled()) {
        NILO_TRACE_MSG("Interrupts not enabled! enabling!\n");	
        osenv_intr_enable();
    }
    
    /* Store the device info in the global data structure. We have to pick
       one, so for now we have a FIXME: default to the first and probably
       only one */
    memcpy ((int8_t *)&client_info->nic, (int8_t *)&devs[0], sizeof(struct etherdev));
    client_info->macaddr[0] = client_info->nic.haddr[0];
    client_info->macaddr[1] = client_info->nic.haddr[1];
    client_info->macaddr[2] = client_info->nic.haddr[2];
    client_info->macaddr[3] = client_info->nic.haddr[3];
    client_info->macaddr[4] = client_info->nic.haddr[4];
    client_info->macaddr[5] = client_info->nic.haddr[5];

    NILO_TRACE_FUNCTION_RETURN;
    return (1);
}

int
currticks()
{
        return ticks;
}

void
bumpticks(void)
{
	ticks++;
}

u_int32_t
getxid(void)
{
    return ((u_int32_t)getpid());
}

void
store(char *d, char *s, int n)
{
    NILO_TRACE_FUNCTION;

    NILO_TRACE_MSG ("Storing %d bytes at %08x\n", n, d);

    NILO_TRACE_FUNCTION_RETURN;
}

int
send_request(client_data_t *client, dhcp_t *pkt, int32_t size,
             int32_t retry, int32_t maxtimeout)
{
    oskit_error_t err;
    int32_t ret;
    struct iphdr *ip;
    struct udphdr *udp;
    u_int32_t i, myip = 0;
    static int32_t starttime = 0;
    client_states_t state;
  
    NILO_TRACE_FUNCTION;

    /* keep a reference to the eth device for the duration of the bootp! */
    oskit_etherdev_addref (client_info->nic.dev);
    
    client->nic.recv_nio = oskit_netio_create(net_receive, &client->nic);
    if (client->nic.recv_nio == NULL)
        panic("unable to create recv_nio");
    oskit_etherdev_getaddr(client->nic.dev, client->nic.haddr);
    
    err = oskit_etherdev_getinfo(client->nic.dev, &client->nic.info);
    if (err)
        panic("error getting info from ethercard");

    NILO_TRACE_MSG("Sending a DHCP packet from the %s card (%x:%x:%x:%x:%x:%x)...\n",
           client->nic.info.name,
           client->nic.haddr[0], client->nic.haddr[1],
           client->nic.haddr[2],
           client->nic.haddr[3], client->nic.haddr[4],
           client->nic.haddr[5]);

    client->bpr = 0;
    
    if (!starttime)
        starttime = currticks();

    while (!client->bpr && retry--) {
        int timeout;
        
        pkt->bp_secs = htons((currticks() - starttime) /
                           (float)BOOTP_TIMER_FREQ);
        
        /* store the current state */
        state = client->client_state;
        /* send DHCP request */
        dhcp_udp_broadcast(pkt->bp_chaddr, 0, 0, BOOTP_SERVER_PORT,
                            sizeof(dhcp_t), pkt);
        
        /* add a delay here between retransmits */
        i = currticks();
// wait for a state change ?
//        while (i+3000 < currticks() && state == client->client_state) {}
//        while (i+1000 < currticks()) {}

        timeout = currticks() + maxtimeout;
        while (timeout > currticks())
            if (client->bpr)
                return 0;       /* success */
    }
    
    return client->bpr ? 0 : OSKIT_ETIMEDOUT;

    NILO_TRACE_FUNCTION_RETURN;
}

static void
dhcp_handler(long xid)
{
    int32_t snaplen;
    
    NILO_TRACE_FUNCTION;
    
    fill_dhcp_header (&bp_sndpkt, client_info->macaddr, client_info->xid);

    switch (client_info->client_state) {
    case REBOOTING:
        fill_dhcp_discover (&bp_sndpkt);
        client_info->client_state = DISCOVERING;
        break;
    case OFFERING:
        fill_dhcp_offer (&bp_sndpkt);
        client_info->client_state = OFFERING;
        break;
    case REQUESTING:
        fill_dhcp_request (&bp_sndpkt);
        client_info->client_state = REQUESTING;
        break;
    case ACKING:
        fill_dhcp_ack (&bp_sndpkt);
        client_info->client_state = ACKING;
        break;
    }
    
    send_request(client_info, &bp_sndpkt, sizeof(bp_sndpkt), 3, 100);

    NILO_TRACE_FUNCTION_RETURN;
}

static void
tftp_handler(u_int32_t server, u_int16_t opcode,
             const char *filename, int32_t blockno)
{
    int	snaplen;

    NILO_TRACE_FUNCTION;

    while (client_info->client_state != TFTP_DONE) {
        switch (client_info->client_state) {
            /* If we've finishd the other two major states, make a
               request for the file. */
        case DHCP_DONE:
        case ARP_DONE:
            client_info->client_state = TFTP_REQUEST;
            NILO_TRACE_MSG("Requesting file %s via tftp.\n", filename);
            snaplen = fill_tftp_request(&tftp_pkt, TFTP_RRQ, filename,
                                        0);
            
            send_tftp_request(client_info, &tftp_pkt, snaplen, 3, 100);
            client_info->client_state = TFTP_LOADING;
            break;
        case TFTP_LOADING:
//            while (client_info->client_state != TFTP_ACKING) {}
        case TFTP_ACKING:
            client_info->client_state = TFTP_ACKING;
            NILO_TRACE_MSG("Ack of tftp block 0x%02x\n",
                           client_info->tftp_blockno);
            snaplen = fill_tftp_request(&tftp_pkt, TFTP_ACK, filename,
                                        client_info->tftp_blockno++);
            
            send_tftp_request(client_info, &tftp_pkt, snaplen, 3, 100);
            while (client_info->client_state == TFTP_ACKING) {}
//            client_info->client_state = TFTP_LOADING;
            break;
         }
        
    }
    
    NILO_TRACE_FUNCTION_RETURN;
}

struct client_data *
init_protocol_code(void) 
{
    struct client_data *client;
    
    if ((client = malloc(sizeof(*client))) == 0)
    {
        NILO_TRACE_MSG("Malloc failed\n");
        exit (1);
    }
    client->bpr = &bp_rcvpkt;
    client->dhcp_handler = (void *)dhcp_handler;
    client->tftp_handler = (void *)tftp_handler;
    return (client);
}

/* Convert an internet number to it's ascified IP version */
char *
_inet_ntoa(unsigned long a) 
{
    struct in_addr i;
    
    //    NILO_TRACE_FUNCTION;
    
    i.s_addr = a;
    return (inet_ntoa(i));
    
    // NILO_TRACE_FUNCTION_RETURN;
}


oskit_error_t
net_receive(void *data, oskit_bufio_t *b, oskit_size_t pkt_size)
{
    oskit_error_t rval = 0;
    unsigned char bcastaddr[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
    static unsigned char *frame;
    struct ether_header *eth;
    int err;
    struct etherdev *dev = (struct etherdev *)data;
    dhcp_t *dhcp_reply;
    tftp_t *tftp_reply;
    struct iphdr *ip;
    struct udphdr *udp;
    arp_t  *arp_reply;
    ip4_t temp;

    NILO_TRACE_FUNCTION;

    if (pkt_size < ETH_MIN_PACKET || pkt_size > ETH_MAX_PACKET) {
        NILO_TRACE_MSG ("ERROR: %s: bad packet size %d\n", dev->info.name,
                        pkt_size);
        rval = OSKIT_E_DEV_BADPARAM;
        goto done;
    }
    
    err = oskit_bufio_map(b, (void **)&frame, 0, pkt_size);
    assert(err == 0);
    
    eth  = (struct ether_header *)frame;
    
    if (memcmp(eth->ether_dhost, &dev->haddr, sizeof dev->haddr) != 0
        && memcmp(eth->ether_dhost, bcastaddr, sizeof bcastaddr) != 0) 
        goto done;			/* not for me */
    
    /* See if it's an ARP reply */
    arp_reply = (arp_t *)&frame[ETHER_HDR_SIZE];
    if (ntohs(arp_reply->hwtype) == 1) {
        NILO_TRACE_MSG ("Got an ARP packet\n");
        memcpy(&temp, arp_reply->sipaddr, sizeof(temp));
        add_to_cache(temp, arp_reply->shwaddr,
                     ARP_SERVER);
#ifdef NILO_USE_TRACING
        dump_arp_packet (arp_reply);
        dump_arp_table (&arptable);
#endif
        if (client_info->client_state == ARPING)
            client_info->client_state = ARP_DONE;
    }

    /* Make sure it is IP  */
    if ((pkt_size < (ETHER_HDR_SIZE + sizeof(struct iphdr) +
                     sizeof(struct udphdr))) ||
        (((frame[12] << 8) | frame[13]) != ETHERTYPE_IP))
        goto done;
    
    ip = (struct iphdr *)&frame[ETHER_HDR_SIZE];
    if ((ip->verhdrlen != 0x45) ||
        dhcp_ipchksum(ip, sizeof(struct iphdr)) ||
        (ip->protocol != IP_UDP))
        goto done;
    
    udp = (struct udphdr *)&frame[ETHER_HDR_SIZE + sizeof(struct iphdr)];
    
    /* DHCP ? */
//        dhcp_reply = (dhcp_t *)&frame[ETHER_HDR_SIZE];
    dhcp_reply = (dhcp_t *)&frame[ETHER_HDR_SIZE + sizeof(struct iphdr)
                                 + sizeof(struct udphdr)];
    
//        client_info->bpr = dhcp_reply;
    
    if ((pkt_size >= (ETHER_HDR_SIZE + sizeof(dhcp_t) + sizeof(struct iphdr)
                      + sizeof(struct udphdr))) &&
        (ntohs(udp->dest) ==  BOOTP_CLIENT_PORT) &&
        (dhcp_reply->bp_opcode == BOOTP_REPLY)) {
        
        /* This is the packet we want */
        err = oskit_bufio_unmap(b, (void **)&frame, 0, pkt_size);
        assert(err == 0);
        
        /* FIXME: This is wrong for DHCP, but right for BOOTP */
        /* We only decode the first packet we receive. */
        if (dhcp_reply) {
            memcpy(client_info->macaddr, 
                   eth->ether_shost, sizeof eth->ether_shost);
            client_info->flags |= BOOTP_NET_SERVER_ADDR;
            dhcp_decode_packet(dhcp_reply, dev->info);
        }
        
    } else {   /* else not DHCP reply */
        if (ntohs(udp->dest) == client_info->tftp_client_port) {
            /* FIXME: ugly, use a void pointer */
            tftp_decode_packet((tftp_t *)dhcp_reply, pkt_size);
        } else {
            NILO_TRACE_MSG ("Unknown packet on port 0x%02x\n",
                            ntohs(udp->dest));
        }
    }
    
 done:
    NILO_TRACE_FUNCTION_RETURN;
    
    return rval;
}

/* This sends a packet on the wire */
int
dhcp_eth_transmit(char *src,           /* source ethernet address */
                  char *dest,          /* destination ethernet address */
                  unsigned short type, /* ether type */
                  unsigned short size,
                  void *buf)
{
    oskit_bufio_t *b;
    oskit_error_t err = OSKIT_E_DEV_IOERR;
    int offset = 0;
    
    /* allocate a buffer to send */
    b = oskit_bufio_create(6+6+sizeof(type) + size);
    if (b != NULL) {
        oskit_size_t actual;
        
        /* fill in the ethernet packet */
        oskit_bufio_write(b, dest, offset, 6, &actual);
        assert(actual == 6);
        offset += 6;
        
        /* fill in source hardware address */
        oskit_bufio_write(b, src, offset, 6, &actual);
        assert(actual == 6);
        offset += 6;
        
        type = htons(type);
        oskit_bufio_write(b, &type, offset, sizeof(type), &actual);
        assert(actual == sizeof(type));
        offset += sizeof(type);
        
        oskit_bufio_write(b, buf, offset, size, &actual);
        assert(actual == size);
        offset += size;

        /* okay, `push' the packet onto the wire */
        err = oskit_netio_push(client_info->nic.send_nio, b, 6+6+sizeof(type) + size);
        
        oskit_bufio_release(b);
    }
    
    return err;
}

/* Decodes the bootp packet & RFC1048 header.
   Fills in bootp_info, setting appropriate flags */
int
dhcp_decode_packet(dhcp_t *dhcp_pkt)
{
    u_int8_t *p = dhcp_pkt->bp_options;
    char s[DHCP_SNAME_LEN];
    int i, j, len, endtag, num;
        
    NILO_TRACE_FUNCTION;

#ifdef NILO_USE_TRACING
    dump_dhcp_packet (dhcp_pkt);
#endif /* NILO_USE_TRACING */

//    client_info->bp_opcode = dhcp_pkt->bp_opcode;
    client_info->htype = dhcp_pkt->bp_htype;
    client_info->xid = dhcp_pkt->bp_xid;
    client_info->flags = dhcp_pkt->bp_flags;

    if ((u_int32_t)dhcp_pkt->bp_ciaddr.num != 0)
        client_info->ciaddr.num = dhcp_pkt->bp_ciaddr.num;

    if ((u_int32_t)dhcp_pkt->bp_yiaddr.num != 0)
        client_info->yiaddr.num = dhcp_pkt->bp_yiaddr.num;
//    client_info->siaddr = (ip4_t)dhcp_pkt->bp_siaddr.num;

    if ((u_int32_t)dhcp_pkt->bp_giaddr.num != 0)
        client_info->giaddr.num = dhcp_pkt->bp_giaddr.num;

    memcpy ((u_int8_t *)client_info->macaddr, dhcp_pkt->bp_chaddr,
        sizeof(dhcp_pkt->bp_chaddr));

//    client_info->macaddr[0] = dhcp_pkt->bp_chaddr[0];
//    client_info->macaddr[1] = dhcp_pkt->bp_chaddr[1];
//    client_info->macaddr[2] = dhcp_pkt->bp_chaddr[2];
//    client_info->macaddr[3] = dhcp_pkt->bp_chaddr[3];
//    client_info->macaddr[4] = dhcp_pkt->bp_chaddr[4];
//    client_info->macaddr[5] = dhcp_pkt->bp_chaddr[5];

    if (*dhcp_pkt->bp_sname && isascii(*dhcp_pkt->bp_sname))
        strcpy (client_info->sname, dhcp_pkt->bp_sname);
    
    if (*dhcp_pkt->bp_file && isascii(*dhcp_pkt->bp_file))
        strcpy (client_info->file,  dhcp_pkt->bp_file);

    /* Look for the last 0xff in the option field, cause it's the
       end tag */
    endtag = DHCP_MAX_OPT_SIZE;
    while (dhcp_pkt->bp_options[endtag--] != TAG_END) {    }

    p = dhcp_pkt->bp_options;
    i = 4;                              /* ignore the vendor magic */

    while (p[i] != TAG_END && p[i++] != 0) {
        len = p[i++];
        switch (p[i-2]) {               /* check the option number */
        case DHCP_IPADDR_LEASE_TIME:
            memcpy (&client_info->leasetime, (int *)(p+i), len);
            /* Swap from network byte order to host */
            client_info->leasetime = ntohl (client_info->leasetime);
            NILO_TRACE_MSG ("Set the leasetime to 0x%04x\n",
                            client_info->leasetime);
            break;
        case DHCP_HOSTNAME:
        case DHCP_DOMAINNAME:
        case DHCP_CLIENT_IDENTIFIER:
            memcpy (s, (char *)p+i, len);
            s[len] = '\0';
            NILO_TRACE_PRINTF ("%s\n", s);
            break;
        case DHCP_SUBNET_MASK:
            memcpy (&client_info->mask.num, (int *)(p+i), len);
            NILO_TRACE_MSG ("Set the subnet mask to %s\n",
                            _inet_ntoa(client_info->mask.num));
            break;
        case DHCP_ROUTERS_ON_SUBNET:
            memcpy (&client_info->routers.num, (int *)(p+i), len);
            NILO_TRACE_MSG ("Set the list of routers to %s\n",
                            _inet_ntoa(client_info->routers.num));
            break;
        case DHCP_SERVER_IDENTIFIER:
            memcpy (&client_info->siaddr.num, (int *)(p+i), len);
            NILO_TRACE_MSG ("Set the Server address to %s\n",
                            _inet_ntoa(client_info->siaddr.num));
            break;
        case DHCP_MESSAGE_TYPE:
            switch (p[i]) {
            case DHCP_DISCOVER:
                break;
            case DHCP_OFFER:
                client_info->client_state = REQUESTING;
//                fill_dhcp_offer (&bp_sndpkt);
//                client_info->client_state = OFFERING;
                break;
            case DHCP_REQUEST:
                break;
            case DHCP_DECLINE:
                break;
            case DHCP_ACK:
               /* Setup the ARP tables. Initialize to zero the 
                hardware address that we'll fill in later when
                we do an ARP request. We know our own though. */
                add_to_cache(client_info->yiaddr,
                             client_info->macaddr, ARP_CLIENT);
                add_to_cache(client_info->siaddr, "\0\0\0\0\0\0",
                             ARP_SERVER);
                add_to_cache(client_info->giaddr, "\0\0\0\0\0\0",
                             ARP_GATEWAY);
                client_info->client_state = DHCP_DONE;
                break;
            case DHCP_NAK:
                break;
            case DHCP_RELEASE:
                break;
            case DHCP_INFORM:
                break;
            }
            break;
        default:
            NILO_TRACE_MSG ("ERROR: Unparsed option 0x%02x!\n", p[i]);
        }
        i+=len;
    }

    NILO_TRACE_FUNCTION_RETURN;
}



/* FIXME: merge this into send_request. */
int
send_tftp_request(client_data_t *client, tftp_t *pkt, int32_t size,
                  int32_t retry, int32_t maxtimeout)
{
    oskit_error_t err;
    int32_t ret;
    struct iphdr *ip;
    struct udphdr *udp;
    u_int32_t i, myip = 0;
    static int32_t starttime = 0;
    client_states_t state;
  
    NILO_TRACE_FUNCTION;

    /* keep a reference to the eth device for the duration of the bootp! */
    oskit_etherdev_addref (client_info->nic.dev);
    
    client->nic.recv_nio = oskit_netio_create(net_receive, &client->nic);
    if (client->nic.recv_nio == NULL)
        panic("unable to create recv_nio");
    oskit_etherdev_getaddr(client->nic.dev, client->nic.haddr);

    err = oskit_etherdev_getinfo(client->nic.dev, &client->nic.info);
    if (err)
        panic("error getting info from ethercard");

    NILO_TRACE_MSG("Sending a TFTP packet from the %s card (%x:%x:%x:%x:%x:%x)...\n",
           client->nic.info.name,
           client->nic.haddr[0], client->nic.haddr[1],
           client->nic.haddr[2],
           client->nic.haddr[3], client->nic.haddr[4],
           client->nic.haddr[5]);

    client->bpr = 0;
    
    if (!starttime)
        starttime = currticks();

    while (!client->bpr && retry--) {
        int timeout;
        
//        pkt->bp_secs = htons((currticks() - starttime) /
//                             (float)100);
        
        /* send TFTP request */
        if (client->tftp_client_port == 0)
            client->tftp_client_port = TFTP_CLIENT_PORT;
        else
            if (client->client_state != TFTP_ACKING &&
                client->client_state != TFTP_LOADING)
                client->tftp_client_port++;

        /* store the current state */
//        state = client->client_state;

#ifdef NILO_USE_TRACING
        dump_tftp_packet (pkt);
#endif

        tftp_udp_transmit(client->macaddr, client->yiaddr.num,
                          client->siaddr.num, client->mask.num,
                          client->tftp_client_port,
                          TFTP_SERVER_PORT, size, pkt);
       
        timeout = currticks() + maxtimeout;

        /* add a delay here between retransmits */
        i = currticks();
// wait for a state change ?
//        while (i+3000 < currticks() && state == client->client_state) {}

        while (timeout > currticks())
//            if (client->bpr)
                return 0;       /* success */
    }
    
    return client->bpr ? 0 : OSKIT_ETIMEDOUT;

    NILO_TRACE_FUNCTION_RETURN;
}


