/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef __NILO_H__
#define __NILO_H__

/* This is the include generated by configuring */
#include "niloconfig.h"

/* Define the PXE types */
#include "pxe_types.h"

/* Define the BOOTP data structures */
#include "bootp.h"

/* Define the DHCP data structures */
#include "dhcp.h"

/* Define the TFTP data structures */
#include "tftp.h"

/* Define the network and NIC related data structures */
#include "net.h"

/* These should be used where sizes are important, e.g. protocol
   packets. */

/* Stolen from netboot */
#define ETH_ALEN        6

/* This is the default timeout value for all ethernet packet
   transmissions */
#define DEFAULT_TIMEOUT 100

typedef enum {
    /* DHCP client states */
    REBOOTING,
    DISCOVERING,
    ACKING,
    REQUESTING,
    OFFERING,
    DHCP_DONE,
    ARPING,
    ARP_DONE,
    /* TFTP client states */
    TFTP_REQUEST,
    TFTP_ACKING,
    TFTP_LOADING,
    TFTP_DONE,
    EXCEPTION = -1
} client_states_t;

/* Store one copy of all the connection data globally */
typedef struct client_data
{	
    u_int32_t flags;		        /* which fields are valid */

    /* Here we store globallay a copy of the data as found through
     all of our DHCP requests. (dhcpdiscover, dhcpoffer) */
    u_int8_t htype;		        /* hardware addr type */
    u_int32_t xid;		        /* transaction ID */
    u_int32_t leasetime;                /* DHCP lease time */
    ip4_t mask;                         /* subnet mask */
    ip4_t routers;                      /* routers on the net */
//    char hostname[BP_SNAME_LEN];        /* hostname */
//    char domainname[BP_SNAME_LEN];      /* domainname */

    ip4_t ciaddr;		        /* client IP address */
    ip4_t yiaddr;		        /* your IP address */
    ip4_t siaddr;		        /* server IP address */
    ip4_t giaddr;		        /* gateway IP address */
    u_int8_t macaddr[ETH_ALEN];         /* client hardware address */
    char sname[BP_SNAME_LEN];           /* server host name */
    char file[BP_FILE_LEN];	        /* boot file name */
    dhcp_t *bpr;                       /* pointer to bootp reply */

    struct device *dev;                 /* FIXME: where does this come from ? */

    struct etherdev nic;                /* a handle for the NIC */
    int32_t ndev;                       /* the number of devices */
    
    void (*bootp_handler)(u_int32_t xid,
                          void (*callback)(int32_t, bootp_t *));
    void (*dhcp_handler)(u_int32_t xid);
    void (*tftp_handler)(u_int32_t, int16_t opcode,
                         const char *filename, int32_t blockno);
    int32_t (*wait_for_work)(void);
    u_int32_t version;
    u_int16_t tftp_blksize;
    u_int16_t tftp_blockno;
    u_int16_t tftp_client_port;
    client_states_t client_state;         /* the state of the client */
} client_data_t;

#define INTSWAP(x)      ((u_int16_t)(((u_int16_t)(x)>>8)|((u_int16_t)(x)<<8)))
#
#define SWAP_TARGET_AND_HOST(buffer,len) \
  do {                                   \
     char tmp;                           \
     char *p = (char *)(buffer);         \
     char *q = ((char *)(buffer)) + len - 1; \
     for (; p < q; p++, q--) {           \
         tmp = *q;                       \
         *q = *p;                        \
         *p = tmp;                       \
         }                               \
     }                                   \
  }                                      \
  while (0)

extern struct client_data *client_info;

extern void init_system(void);
extern client_data_t *init_protocol_code(void);
extern int32_t init_hardware(client_data_t *);
extern u_int32_t getxid(void);
extern void store(char *d, char *s, int n);

#endif /* __NILO_H__ */
