/* 
   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
 * The PXENV structure is being depreciated by Intel, and will not
 * appear in the next version of Intel's PXE specifitcation. This
 * PXENV support is mostly for backward compatability. 
 *
 * As the PXE compliance in Nilo is for newer ROMS, we don't really
 * want to support the older PXENV based ROMS, but we'll try to get
 * as close to conformance to the older spec as possible.
 */

#ifndef __PXENV_H__
#define __PXENV_H__

/* This is generated by running configure */
#include "niloconfig.h"

/* Get the default definitions  */
#include "pxe.h"

typedef struct s_PXENV {
    char signature[6];		/* "PXENV+" */
    u_int16_t version;		/* PXE version number.  LSB is minor */
				/* version.  MSB is major version. */
    u_int8_t struct_length;	/* Length of PXE-2.0 Entry Point */
				/* structure in bytes. */
    u_int8_t struct_cksum;	/* Used to make structure checksum */
				/* equal zero. */
    u_int32_t rmentry;		/* Real mode API entry point  */
				/* segment:offset. */
    u_int16_t pmentry_off;	/* Protected mode API entry point */
    u_int32_t pmentry_seg;	/* segment:offset. This will always */
				/* be zero. Protected mode API calls */
				/* must be made through the API entry */
				/* points in the PXE Runtime ID */
				/* structure. */
    u_int16_t stackseg;		/* Real mode stack segment. */
    u_int16_t stacksize;	/* Stack segment size in bytes. */
    u_int16_t basecode_seg;	/* Real mode base-code code segment. */
    u_int16_t basecode_size;	/* Base-code code segment size */
    u_int16_t basedata_seg;	/* Real mode base-code data segment. */
    u_int16_t basedata_size;	/* Base-code data segment size */
    u_int16_t undidata_seg;	/* Real mode UNDI data segment. */
    u_int16_t undidata_size;	/* UNDI data segment size in bytes. */
    u_int16_t undicode_seg;	/* Real mode UNDI code segment. */
    u_int16_t undicode_size;	/* UNDI code segment size in bytes. */
    pxe_t  *runtimeptr;         /* Real mode segment:offset pointer */
				/* to PXE Runtime ID structure. */
} pxenv_t;

/* Generic API errors - these do not match up with the M0x or E0x messages
 * that are reported by the loader.
 */
#define PXENV_STATUS_SUCCESS                            0x00
#define PXENV_STATUS_FAILURE                            0x01
#define PXENV_STATUS_BAD_FUNC                           0x02
#define PXENV_STATUS_UNSUPPORTED                        0x03
#define PXENV_STATUS_KEEP_UNDI                          0x04
#define PXENV_STATUS_KEEP_ALL                           0x05

#if 0
/* Initialization routine errors (0x00 to 0x0F) */
#define PXENV_STATUS_INIT_
#endif

/* ARP errors (0x10 to 0x17) */
#define PXENV_STATUS_ARP_TIMEOUT                        0x11

/* Base-Code state errors */
#define PXENV_STATUS_UDP_CLOSED                         0x18
#define PXENV_STATUS_UDP_OPEN                           0x19
#define PXENV_STATUS_TFTP_CLOSED                        0x1A
#define PXENV_STATUS_TFTP_OPEN                          0x1B

/* BIOS/system errors (0x20 to 0x2F) */
#define PXENV_STATUS_MCOPY_PROBLEM                      0x20
#define PXENV_STATUS_BIS_INTEGRITY_FAILURE              0x21
#define PXENV_STATUS_BIS_VALIDATE_FAILURE               0x22
#define PXENV_STATUS_BIS_INIT_FAILURE                   0x23
#define PXENV_STATUS_BIS_SHUTDOWN_FAILURE               0x24
#define PXENV_STATUS_BIS_GBOA_FAILURE                   0x25
#define PXENV_STATUS_BIS_FREE_FAILURE                   0x26
#define PXENV_STATUS_BIS_GSI_FAILURE                    0x27
#define PXENV_STATUS_BIS_BAD_CKSUM                      0x28

/* (M)TFTP errors (0x30 to 0x4F) */
#define PXENV_STATUS_TFTP_CANNOT_ARP_ADDRESS            0x30
#define PXENV_STATUS_TFTP_OPEN_TIMEOUT                  0x32
#define PXENV_STATUS_TFTP_UNKNOWN_OPCODE                0x33
#define PXENV_STATUS_TFTP_READ_TIMEOUT                  0x35
#define PXENV_STATUS_TFTP_ERROR_OPCODE                  0x36
#define PXENV_STATUS_TFTP_CANNOT_OPEN_CONNECTION        0x38
#define PXENV_STATUS_TFTP_CANNOT_READ_FROM_CONNECTION   0x39
#define PXENV_STATUS_TFTP_TOO_MANY_PACKAGES             0x3A
#define PXENV_STATUS_TFTP_FILE_NOT_FOUND                0x3B
#define PXENV_STATUS_TFTP_ACCESS_VIOLATION              0x3C
#define PXENV_STATUS_TFTP_NO_MCAST_ADDRESS              0x3D
#define PXENV_STATUS_TFTP_NO_FILESIZE                   0x3E
#define PXENV_STATUS_TFTP_INVALID_PACKET_SIZE           0x3F

/* reserved errors (0x40 to 0x4F) */

/* BOOTP/DHCP errors (0x50 to 0x5F) */
#define PXENV_STATUS_DHCP_TIMEOUT                       0x51
#define PXENV_STATUS_DHCP_NO_IP_ADDRESS                 0x52
#define PXENV_STATUS_DHCP_NO_BOOTFILE_NAME              0x53
#define PXENV_STATUS_DHCP_BAD_IP_ADDRESS                0x54

/* Driver errors (0x60 to 0x6F) */
/* These errors are for UNDI compatible NIC drivers. */
#define PXENV_STATUS_UNDI_INVALID_FUNCTION              0x60
#define PXENV_STATUS_UNDI_MEDIATEST_FAILED              0x61
#define PXENV_STATUS_UNDI_CANNOT_INIT_NIC_FOR_MCAST     0x62
#define PXENV_STATUS_UNDI_CANNOT_INITIALIZE_NIC         0x63
#define PXENV_STATUS_UNDI_CANNOT_INITIALIZE_PHY         0x64
#define PXENV_STATUS_UNDI_CANNOT_READ_CONFIG_DATA       0x65
#define PXENV_STATUS_UNDI_CANNOT_READ_INIT_DATA         0x66
#define PXENV_STATUS_UNDI_BAD_MAC_ADDR                  0x67
#define PXENV_STATUS_UNDI_BAD_EEPROM_CKSUM              0x68
#define PXENV_STATUS_UNDI_ERROR_SETTING_ISR             0x69
#define PXENV_STATUS_UNDI_INVALID_STATE                 0x6A

/* Bootstrap (.1) errors (0x70 to 0x7F) */
/* These errors are for the bootstrap layer. */
#define PXENV_STATUS_BSTRAP_PROMPT_MENU                 0x74
#define PXENV_STATUS_BSTRAP_MCAST_ADDR                  0x76
#define PXENV_STATUS_BSTRAP_MISSING_LIST                0x77
#define PXENV_STATUS_BSTRAP_NO_RESPONSE                 0x78
#define PXENV_STATUS_BSTRAP_FILE_TOO_BIG                0x79

/* Environment (.2) errors (0x80 to 0x8F) */
/* These errors are for environment layers. */

/* reserved errors (0x90 to 0x9F) */

/* Misc errors (0xA0 to 0xAF) */
#define PXENV_STATUS_CANCELED_BY_KEYSTROKE              0xA0
#define PXENV_STATUS_BINL_NO_PXE_SERVER                 0xA1
#define PXENV_STATUS_NOT_AVAILABLE_IN_PMODE             0xA2
#define PXENV_STATUS_NOT_AVAILABLE_IN_RMODE             0xA3

/* BUSD errors (0xB0 to 0xBF) */
#define PXENV_STATUS_BUSD_BUS_NOT_ENABLED               0xB0
#define PXENV_STATUS_BUSD_DEVICE_NOT_ENABLED            0xB1
#define PXENV_STATUS_BUSD_NO_ROMID                      0xB2
#define PXENV_STATUS_BUSD_BAD_ROMID                     0xB3

/* BC/UNDI Loader errors (0xC0 to 0xCF) */
#define PXENV_STATUS_LOADER_NO_FREE_BASE_MEMORY         0xC0
#define PXENV_STATUS_LOADER_NO_BC_ROMID                 0xC1
#define PXENV_STATUS_LOADER_BAD_BC_ROMID                0xC2
#define PXENV_STATUS_LOADER_BAD_BC_RUNTIME_IMAGE        0xC3
#define PXENV_STATUS_LOADER_NO_UNDI_ROMID               0xC4
#define PXENV_STATUS_LOADER_BAD_UNDI_ROMID              0xC5
#define PXENV_STATUS_LOADER_BAD_UNDI_DRIVER_IMAGE       0xC6
#define PXENV_STATUS_LOADER_NO_PXE_STRUCT               0xC8
#define PXENV_STATUS_LOADER_NO_PXENV_STRUCT             0xC9
#define PXENV_STATUS_LOADER_UNDI_START                  0xCA
#define PXENV_STATUS_LOADER_BC_START                    0xCB

/* Vendor errors (0xD0 to 0xFF) */

#endif  /* __PXENV_H__ */

